#include "ahg-item.h"

enum {
    PROP_0,
};

enum {
    CHANGED,
    UNKNOWN,
    LAST_SIGNAL,
};

struct _AhgItemPrivate {
    AhgItemType type;

    char *uri;
    char *mimetype;
    GHashTable *metadata;

    guint32 change_id;
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), AHG_TYPE_ITEM, AhgItemPrivate))
G_DEFINE_TYPE (AhgItem, ahg_item, G_TYPE_OBJECT);
static guint32 signals[LAST_SIGNAL] = {0,};

static void
ahg_item_finalize (GObject *object)
{
    AhgItem *self = (AhgItem *) object;
    AhgItemPrivate *priv = self->priv;

    if (priv->uri) {
        g_free (priv->uri);
    }

    if (priv->mimetype) {
        g_free (priv->mimetype);
    }

    G_OBJECT_CLASS (ahg_item_parent_class)->finalize (object);
}

static void
ahg_item_dispose (GObject *object)
{
    AhgItem *self = (AhgItem *) object;
    G_OBJECT_CLASS (ahg_item_parent_class)->dispose (object);
}

static void
ahg_item_set_property (GObject      *object,
                       guint         prop_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
    AhgItem *self = (AhgItem *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahg_item_get_property (GObject    *object,
                       guint       prop_id,
                       GValue     *value,
                       GParamSpec *pspec)
{
    AhgItem *self = (AhgItem *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahg_item_class_init (AhgItemClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;

    o_class->dispose = ahg_item_dispose;
    o_class->finalize = ahg_item_finalize;
    o_class->set_property = ahg_item_set_property;
    o_class->get_property = ahg_item_get_property;

    g_type_class_add_private (klass, sizeof (AhgItemPrivate));

    signals[CHANGED] = g_signal_new ("changed", G_TYPE_FROM_CLASS (o_class),
                                     G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
                                     0, NULL, NULL,
                                     g_cclosure_marshal_VOID__VOID,
                                     G_TYPE_NONE, 0);
    signals[UNKNOWN] = g_signal_new ("unknown", G_TYPE_FROM_CLASS (o_class),
                                     G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
                                     0, NULL, NULL,
                                     g_cclosure_marshal_VOID__VOID,
                                     G_TYPE_NONE, 0);
}

static void
ahg_item_init (AhgItem *self)
{
    AhgItemPrivate *priv = GET_PRIVATE (self);

    self->priv = priv;
    priv->metadata = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            g_free, g_free);
}

AhgItem *
ahg_item_new (const char *uri)
{
    AhgItem *item;
    AhgItemPrivate *priv;

    item = g_object_new (AHG_TYPE_ITEM, NULL);
    priv = item->priv;

    priv->uri = g_strdup (uri);

    return item;
}

AhgItem *
ahg_item_new_full (AhgItemType type,
                   const char *uri,
                   const char *mimetype)
{
    AhgItem *item;
    AhgItemPrivate *priv;

    item = ahg_item_new (uri);
    priv = item->priv;

    priv->type = type;
    priv->mimetype = g_strdup (mimetype);

    return item;
}

AhgItemType
ahg_item_get_item_type (AhgItem *item)
{
    AhgItemPrivate *priv = item->priv;

    return priv->type;
}

const char *
ahg_item_get_uri (AhgItem *item)
{
    AhgItemPrivate *priv = item->priv;

    return priv->uri;
}

const char *
ahg_item_get_mimetype (AhgItem *item)
{
    AhgItemPrivate *priv = item->priv;

    return priv->mimetype;
}

const char *
ahg_item_get_metadata (AhgItem    *item,
                       const char *key)
{
    AhgItemPrivate *priv = item->priv;

    return g_hash_table_lookup (priv->metadata, key);
}

static gboolean
emit_change_signal (gpointer data)
{
    AhgItem *item = (AhgItem *) data;
    AhgItemPrivate *priv = item->priv;

    g_signal_emit (item, signals[CHANGED], 0);

    priv->change_id = 0;
    return FALSE;
}

static void
ensure_change_signal (AhgItem *item)
{
    AhgItemPrivate *priv = item->priv;

    if (priv->change_id > 0) {
        return;
    }

    priv->change_id = g_timeout_add (500, emit_change_signal, item);
}

void
ahg_item_set_metadata (AhgItem    *item,
                       const char *key,
                       const char *value)
{
    AhgItemPrivate *priv = item->priv;

    if (value == NULL || *value == '\0') {
        return;
    }

    g_hash_table_insert (priv->metadata,
                         g_strdup (key),
                         g_strdup (value));

    ensure_change_signal (item);
}

void
ahg_item_set_item_type (AhgItem    *item,
                        AhgItemType type)
{
    AhgItemPrivate *priv = item->priv;

    priv->type = type;

    if (type == AHG_ITEM_TYPE_UNKNOWN) {
        g_signal_emit (item, signals[UNKNOWN], 0);
    } else {
        ensure_change_signal (item);
    }
}

void
ahg_item_set_mimetype (AhgItem    *item,
                       const char *mimetype)
{
    AhgItemPrivate *priv = item->priv;

    priv->mimetype = g_strdup (mimetype);
    ensure_change_signal (item);
}
