/*
 * Hornsey - Moblin Media Player.
 * Copyright © 2009 - 2010 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef __AHG_ITEM_H__
#define __AHG_ITEM_H__

#include <glib-object.h>


G_BEGIN_DECLS

#define AHG_TYPE_ITEM                                                   \
   (ahg_item_get_type())
#define AHG_ITEM(obj)                                                   \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj),                                  \
                                AHG_TYPE_ITEM,                          \
                                AhgItem))
#define AHG_ITEM_CLASS(klass)                                           \
   (G_TYPE_CHECK_CLASS_CAST ((klass),                                   \
                             AHG_TYPE_ITEM,                             \
                             AhgItemClass))
#define IS_AHG_ITEM(obj)                                                \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj),                                  \
                                AHG_TYPE_ITEM))
#define IS_AHG_ITEM_CLASS(klass)                                        \
   (G_TYPE_CHECK_CLASS_TYPE ((klass),                                   \
                             AHG_TYPE_ITEM))
#define AHG_ITEM_GET_CLASS(obj)                                         \
   (G_TYPE_INSTANCE_GET_CLASS ((obj),                                   \
                               AHG_TYPE_ITEM,                           \
                               AhgItemClass))

typedef struct _AhgItemPrivate AhgItemPrivate;
typedef struct _AhgItem      AhgItem;
typedef struct _AhgItemClass AhgItemClass;
typedef enum _AhgItemType {
    AHG_ITEM_TYPE_UNDEFINED,
    AHG_ITEM_TYPE_AUDIO,
    AHG_ITEM_TYPE_IMAGE,
    AHG_ITEM_TYPE_VIDEO,
    AHG_ITEM_TYPE_UNKNOWN /* Item is not in Tracker */
} AhgItemType;

#define AHG_ITEM_METADATA_TRACK "track"
#define AHG_ITEM_METADATA_TITLE "title"
#define AHG_ITEM_METADATA_ALBUM "album"
#define AHG_ITEM_METADATA_DATE "date"
#define AHG_ITEM_METADATA_ARTIST "artist"
#define AHG_ITEM_METADATA_EPISODE "episode"
#define AHG_ITEM_METADATA_SEASON "season"
#define AHG_ITEM_METADATA_SERIES_NAME "showname"
#define AHG_ITEM_METADATA_WIDTH "width"
#define AHG_ITEM_METADATA_HEIGHT "height"

struct _AhgItem
{
    GObject parent;

    AhgItemPrivate *priv;
};

struct _AhgItemClass
{
    GObjectClass parent_class;
};

GType ahg_item_get_type (void) G_GNUC_CONST;
AhgItem *ahg_item_new (const char *uri);
AhgItem *ahg_item_new_full (AhgItemType type,
                            const char *uri,
                            const char *mimetype);
AhgItemType ahg_item_get_item_type (AhgItem *item);
const char *ahg_item_get_uri (AhgItem *item);
const char *ahg_item_get_mimetype (AhgItem *item);
const char *ahg_item_get_metadata (AhgItem    *item,
                                   const char *key);
void ahg_item_set_metadata (AhgItem    *item,
                            const char *key,
                            const char *value);
void ahg_item_set_item_type (AhgItem    *item,
                             AhgItemType type);
void ahg_item_set_mimetype (AhgItem    *item,
                            const char *mimetype);


G_END_DECLS

#endif /* __AHG_ITEM_H__ */
