#include <libtracker-client/tracker.h>

#include "ahg-tracker-client.h"

enum {
    PROP_0,
};

enum {
    LAST_SIGNAL,
};

struct _AhgTrackerClientPrivate {
    TrackerClient *client;

    guint32 query_id;
    GPtrArray *pending;
    GHashTable *results;

    /* FIXME: how to use these? */
#if 0
    TrackerLanguage *language;
    GHashTable *stop_words;
#endif

    char *search_string;
    guint32 search_id;
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), AHG_TYPE_TRACKER_CLIENT, AhgTrackerClientPrivate))
G_DEFINE_TYPE (AhgTrackerClient, ahg_tracker_client, G_TYPE_OBJECT);
static guint32 signals[LAST_SIGNAL] = {0,};

#define URI 0
#define MIMETYPE 1
#define DATE 2
#define TITLE 3
#define ARTIST_NAME 4
#define ALBUM_TITLE 5
#define TRACK 6
#define SEASON 7
#define EPISODE 8
#define WIDTH 9
#define HEIGHT 10
#define ITEM_QUERY "SELECT nie:url(?uri) nie:mimeType(?uri) nie:contentCreated(?uri) nie:title(?uri) nmm:artistName(nmm:performer(?uri)) nmm:albumTitle(nmm:musicAlbum(?uri)) nmm:trackNumber(?uri) nmm:season(?uri) nmm:episodeNumber(?uri) nfo:width(?uri) nfo:height(?uri) " \
    "WHERE { "

static void
ahg_tracker_client_finalize (GObject *object)
{
    AhgTrackerClient *self = (AhgTrackerClient *) object;

    G_OBJECT_CLASS (ahg_tracker_client_parent_class)->finalize (object);
}

static void
ahg_tracker_client_dispose (GObject *object)
{
    AhgTrackerClient *self = (AhgTrackerClient *) object;

    G_OBJECT_CLASS (ahg_tracker_client_parent_class)->dispose (object);
}

static void
ahg_tracker_client_set_property (GObject      *object,
                          guint         prop_id,
                          const GValue *value,
                          GParamSpec   *pspec)
{
    AhgTrackerClient *self = (AhgTrackerClient *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahg_tracker_client_get_property (GObject    *object,
                          guint       prop_id,
                          GValue     *value,
                          GParamSpec *pspec)
{
    AhgTrackerClient *self = (AhgTrackerClient *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahg_tracker_client_class_init (AhgTrackerClientClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;

    o_class->dispose = ahg_tracker_client_dispose;
    o_class->finalize = ahg_tracker_client_finalize;
    o_class->set_property = ahg_tracker_client_set_property;
    o_class->get_property = ahg_tracker_client_get_property;

    g_type_class_add_private (klass, sizeof (AhgTrackerClientPrivate));
}

static void
ahg_tracker_client_init (AhgTrackerClient *self)
{
    AhgTrackerClientPrivate *priv = GET_PRIVATE (self);

    self->priv = priv;

    priv->client = tracker_client_new (FALSE, G_MAXINT);
    priv->pending = g_ptr_array_new ();
    priv->results = g_hash_table_new (g_str_hash, g_str_equal);

#if 0
    priv->language = tracker_language_new (NULL);
    priv->stop_words = tracker_language_get_stop_words (priv->language);
#endif
}

static void
update_item (AhgItem *item,
             char   **result)
{
    ahg_item_set_mimetype (item, result[MIMETYPE]);
    ahg_item_set_metadata (item, AHG_ITEM_METADATA_DATE, result[DATE]);
    ahg_item_set_metadata (item, AHG_ITEM_METADATA_TITLE, result[TITLE]);

    if (g_str_has_prefix (result[MIMETYPE], "audio")) {
        ahg_item_set_item_type (item, AHG_ITEM_TYPE_AUDIO);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_ARTIST,
                               result[ARTIST_NAME]);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_ALBUM,
                               result[ALBUM_TITLE]);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_TRACK,
                               result[TRACK]);
    } else if (g_str_has_prefix (result[MIMETYPE], "image")) {
        ahg_item_set_item_type (item, AHG_ITEM_TYPE_IMAGE);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_WIDTH, result[WIDTH]);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_HEIGHT, result[HEIGHT]);
    } else if (g_str_has_prefix (result[MIMETYPE], "video")) {
        ahg_item_set_item_type (item, AHG_ITEM_TYPE_VIDEO);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_SEASON, result[SEASON]);
        ahg_item_set_metadata (item, AHG_ITEM_METADATA_EPISODE,
                               result[EPISODE]);
    }
}

struct _QueryData {
    AhgTrackerClient *client;
    GHashTable *requested;
    char *query;
};

static gboolean
unknown_and_remove (gpointer key,
                    gpointer value,
                    gpointer userdata)
{
    AhgItem *item = (AhgItem *) value;

    ahg_item_set_item_type (item, AHG_ITEM_TYPE_UNKNOWN);
    return TRUE;
}

static void
query_items_async_reply (GPtrArray *results,
                         GError    *error,
                         gpointer   userdata)
{
    struct _QueryData *qd = (struct _QueryData *) userdata;
    AhgTrackerClient *client = qd->client;
    AhgTrackerClientPrivate *priv = client->priv;
    int i;

    if (error) {
        g_warning ("Could not get search results: %s", error->message);
        g_print ("Query was %s\n", qd->query);
        g_error_free (error);

        goto cleanup;
    }

    if (results == NULL) {
        goto cleanup;
    }

    for (i = 0; i < results->len; i++) {
        char **result = results->pdata[i];
        AhgItem *item = g_hash_table_lookup (qd->requested, result[URI]);

        if (item == NULL) {
            g_print ("Could not find item for %s\n", result[URI]);
            continue;
        }

        update_item (item, result);
        g_hash_table_remove (qd->requested, result[URI]);
    }

  cleanup:
    /* Any other requests are unknown to tracker */
    g_hash_table_foreach_remove (qd->requested, unknown_and_remove, NULL);
    g_hash_table_destroy (qd->requested);

    g_free (qd->query);
    g_slice_free (struct _QueryData, qd);
}

static gboolean
query_tracker (gpointer data)
{
    AhgTrackerClient *client = (AhgTrackerClient *) data;
    AhgTrackerClientPrivate *priv = client->priv;
    GString *query_builder;
    int i = 0;

    if (priv->pending->len == 0) {
        priv->query_id = 0;
        return FALSE;
    }

    while (i < priv->pending->len) {
        struct _QueryData *qd = g_slice_new0 (struct _QueryData);
        int j;

        qd->client = client;
        qd->requested = g_hash_table_new (g_str_hash, g_str_equal);

        query_builder = g_string_new (ITEM_QUERY);

        g_string_append_printf (query_builder, "{ ?uri nie:url \"%s\" }",
                                ahg_item_get_uri (priv->pending->pdata[i]));
        g_hash_table_insert (qd->requested,
                             (gpointer) ahg_item_get_uri (priv->pending->pdata[i]),
                             priv->pending->pdata[i]);

        for (j = 1; j < 50 && (i + j) < priv->pending->len; j++) {
            AhgItem *item = priv->pending->pdata[i + j];

            g_hash_table_insert (qd->requested,
                                 (gpointer) ahg_item_get_uri (item), item);
            g_string_append_printf (query_builder,
                                    " UNION { ?uri nie:url \"%s\" }",
                                    ahg_item_get_uri (item));
        }
        g_string_append (query_builder, "}");

        qd->query = query_builder->str;
        tracker_resources_sparql_query_async (priv->client, query_builder->str,
                                              query_items_async_reply, qd);
        g_string_free (query_builder, FALSE);

        i += j;
    }

    g_ptr_array_free (priv->pending, TRUE);
    priv->pending = g_ptr_array_new ();


    priv->query_id = 0;
    return FALSE;
}

static void
ensure_query (AhgTrackerClient *client)
{
    AhgTrackerClientPrivate *priv = client->priv;

    if (priv->query_id > 0) {
        return;
    }

    priv->query_id = g_timeout_add (500, query_tracker, client);
}

AhgItem *
ahg_tracker_client_get_item (AhgTrackerClient *client,
                             const char       *uri)
{
    AhgTrackerClientPrivate *priv = client->priv;
    AhgItem *item = ahg_item_new (uri);

    g_ptr_array_add (priv->pending, item);
    ensure_query (client);

    return item;
}

static char *
split_query_string (AhgTrackerClient *client,
                    const char       *str)
{
    AhgTrackerClientPrivate *priv = client->priv;
    GString *fts;
    char **terms, **no_stop_terms;
    int i, t, len;

    /* FIXME: Should we remove all the punctuation before splitting? */
    terms = g_strsplit (str, G_STR_DELIMITERS, -1);
    len = g_strv_length (terms);

    /* FIXME: how do you get at the stop words? */
#if 0
    no_stop_terms = g_new0 (char *, len);

    for (i = 0, t = 0; terms[i]; i++) {
        if (g_hash_table_lookup (priv->stop_words, terms[i])) {
            /* Found a stop word, removing from search */
            g_free (terms[i]);
            terms[i] = NULL;
        } else {
            no_stop_terms[t] = terms[i];
        }
    }

    /* Don't need to free the individual items */
    g_free (terms);

    len = t;
#else
    no_stop_terms = terms;
#endif

    fts = g_string_new ("");
    for (i = 0; no_stop_terms[i]; i++) {
        g_string_append (fts, no_stop_terms[i]);
        g_string_append_c (fts, '*');
        if (i < len - 1) {
            g_string_append (fts, " ");
        }

        g_free (no_stop_terms[i]);
    }
    g_free (no_stop_terms);

    return g_string_free (fts, FALSE);
}

struct _SearchData {
    AhgTrackerClient *client;
    AhgTrackerSearchCallback callback;
    gpointer userdata;
};

static void
search_items_async_reply (GPtrArray *results,
                          GError    *error,
                          gpointer   userdata)
{
    struct _SearchData *sd = (struct _SearchData *) userdata;

    if (sd->callback) {
        sd->callback (sd->client, results, error, sd->userdata);
    }

    g_slice_free (struct _SearchData, sd);
}

static gboolean
deferred_search (gpointer data)
{
    struct _SearchData *sd = (struct _SearchData *) data;
    AhgTrackerClientPrivate *priv = sd->client->priv;
    char *fts, *query;

    priv->search_id = 0;
    fts = split_query_string (sd->client, priv->search_string);
    g_free (priv->search_string);
    priv->search_string = NULL;

    /* Search for all mimetypes, we will filter them later */
    query = g_strdup_printf ("SELECT nie:url(?uri) nie:mimeType(?uri)"
                             "WHERE { "
                             "   ?uri fts:match \"%s\" ."
                             "}", fts);
    g_free (fts);

    tracker_resources_sparql_query_async (sd->client->priv->client, query,
                                          search_items_async_reply, sd);
    g_free (query);

    return FALSE;
}

void
ahg_tracker_client_search (AhgTrackerClient         *client,
                           const char               *search_string,
                           AhgTrackerSearchCallback  callback,
                           gpointer                  data)
{
    AhgTrackerClientPrivate *priv = client->priv;
    struct _SearchData *sd;

    g_free (priv->search_string);
    priv->search_string = g_strdup (search_string);

    if (priv->search_id > 0) {
        return;
    }

    sd = g_slice_new (struct _SearchData);
    sd->client = client;
    sd->callback = callback;
    sd->userdata = data;

    priv->search_id = g_timeout_add (500, deferred_search, sd);
}
