#include <glib.h>
#include <glib/gi18n.h>

#include "ahg-item.h"
#include "ahoghill-playlist-np.h"

enum {
    PROP_0,
};

enum {
    POS_CHANGED,
    LAST_SIGNAL
};

struct _AhoghillPlaylistNpPrivate {
    MxWidget *primary;
    MxWidget *secondary;

    guint pos_changed_id;
    MxWidget *seekbar;

    /* This is a hack because MxAdjustment does not emit signals sync now */
    gboolean ignore_position_change;
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), AHOGHILL_TYPE_PLAYLIST_NP, AhoghillPlaylistNpPrivate))
G_DEFINE_TYPE (AhoghillPlaylistNp, ahoghill_playlist_np, MX_TYPE_TABLE);

static guint32 signals[LAST_SIGNAL] = {0, };

static void
ahoghill_playlist_np_finalize (GObject *object)
{
    G_OBJECT_CLASS (ahoghill_playlist_np_parent_class)->finalize (object);
}

static void
ahoghill_playlist_np_dispose (GObject *object)
{
    G_OBJECT_CLASS (ahoghill_playlist_np_parent_class)->dispose (object);
}

static void
ahoghill_playlist_np_class_init (AhoghillPlaylistNpClass *klass)
{
    GObjectClass *o_class = (GObjectClass *)klass;

    o_class->dispose = ahoghill_playlist_np_dispose;
    o_class->finalize = ahoghill_playlist_np_finalize;

    g_type_class_add_private (klass, sizeof (AhoghillPlaylistNpPrivate));

    signals[POS_CHANGED] = g_signal_new ("position-changed",
                                         G_TYPE_FROM_CLASS (klass),
                                         G_SIGNAL_RUN_FIRST |
                                         G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                         g_cclosure_marshal_VOID__DOUBLE,
                                         G_TYPE_NONE, 1, G_TYPE_DOUBLE);
}

static void
seek_position_changed (MxSlider           *slider,
                       GParamSpec         *pspec,
                       AhoghillPlaylistNp *np)
{
    AhoghillPlaylistNpPrivate *priv = np->priv;
    double position = mx_slider_get_progress (slider);

    if (priv->ignore_position_change == FALSE) {
        g_signal_emit (np, signals[POS_CHANGED], 0, position);
    }
    priv->ignore_position_change = FALSE;
}

static void
ahoghill_playlist_np_init (AhoghillPlaylistNp *self)
{
    AhoghillPlaylistNpPrivate *priv;

    self->priv = GET_PRIVATE (self);
    priv = self->priv;

    priv->ignore_position_change = FALSE;

    priv->primary = mx_label_new ("");
    mx_stylable_set_style_class ((MxStylable *) priv->primary,
                                 "AhoghillPrimaryLabel");
    mx_table_add_actor (MX_TABLE (self),
                          (ClutterActor *) priv->primary,
                          0, 0);

    priv->secondary = mx_label_new ("");
    mx_stylable_set_style_class ((MxStylable *) priv->secondary,
                                 "AhoghillSecondaryLabel");
    mx_table_add_actor (MX_TABLE (self),
                          (ClutterActor *) priv->secondary,
                          1, 0);

#if 0
    priv->audio_progress = mx_adjustment_new (0.0, 0.0, 1.0, 0.0, 0.0, 0.0);
    priv->pos_changed_id = g_signal_connect
        (priv->audio_progress, "notify::value",
         G_CALLBACK (seek_position_changed), self);
    priv->seekbar = mx_scroll_bar_new (priv->audio_progress);
    mx_stylable_set_style_class ((MxStylable *) priv->seekbar,
                                 "AhoghillProgressBar");
#endif

    priv->seekbar = mx_slider_new ();
    priv->pos_changed_id = g_signal_connect
        (priv->seekbar, "notify::progress",
         G_CALLBACK (seek_position_changed), self);
    clutter_actor_set_size ((ClutterActor *) priv->seekbar, -1, 20);
    /* clutter_actor_set_reactive ((ClutterActor *) priv->seekbar, FALSE); */

    mx_table_add_actor_with_properties (MX_TABLE (self),
                                        (ClutterActor *) priv->seekbar,
                                        2, 0,
                                        "col-span", 1,
                                        "x-align", 0.0,
                                        NULL);
}

void
ahoghill_playlist_np_set_item (AhoghillPlaylistNp *now_playing,
                               AhgItem            *item)
{
    AhoghillPlaylistNpPrivate *priv = now_playing->priv;
    const char *title = NULL, *artist = NULL;
    char *primary = NULL;

    if (item == NULL) {
        clutter_actor_set_reactive ((ClutterActor *) priv->seekbar, FALSE);

        /* FIXME: Fade out text? */
        mx_label_set_text ((MxLabel *) priv->primary, "");
        mx_label_set_text ((MxLabel *) priv->secondary, "");
        return;
    }

    clutter_actor_set_reactive ((ClutterActor *) priv->seekbar, TRUE);

    switch (ahg_item_get_item_type (item)) {
    case AHG_ITEM_TYPE_AUDIO:
        title = ahg_item_get_metadata (item, AHG_ITEM_METADATA_TITLE);
        artist = ahg_item_get_metadata (item, AHG_ITEM_METADATA_ARTIST);
        break;

    default:
        /* We don't need to handle non audio */
        return;
    }

    if (title == NULL) {
        char *basename;
        basename = g_path_get_basename (ahg_item_get_uri (item));
        primary = g_uri_unescape_string (basename, NULL);
        g_free (basename);
    }

    mx_label_set_text (MX_LABEL (priv->primary), title ? title : primary);
    mx_label_set_text (MX_LABEL (priv->secondary), artist ? artist : "");

    g_free (primary);
}

void
ahoghill_playlist_np_set_position (AhoghillPlaylistNp *now_playing,
                                   double              position)
{
    AhoghillPlaylistNpPrivate *priv = now_playing->priv;

    priv->ignore_position_change = TRUE;
    mx_slider_set_progress ((MxSlider *) priv->seekbar, position);
}

