#include <glib/gi18n.h>

#include "ahg-item.h"
#include "ahoghill-queue-tile.h"

enum {
    PROP_0,
    PROP_ITEM,
};

enum {
    CLICKED,
    LAST_SIGNAL
};

struct _AhoghillQueueTilePrivate {
    AhgItem *item;
    MxWidget *row1;
    MxWidget *row2;
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), AHOGHILL_TYPE_QUEUE_TILE, AhoghillQueueTilePrivate))
G_DEFINE_TYPE (AhoghillQueueTile, ahoghill_queue_tile, MX_TYPE_TABLE);
static guint32 signals[LAST_SIGNAL] = {0,};

static void
ahoghill_queue_tile_finalize (GObject *object)
{
    G_OBJECT_CLASS (ahoghill_queue_tile_parent_class)->finalize (object);
}

static void
ahoghill_queue_tile_dispose (GObject *object)
{
    G_OBJECT_CLASS (ahoghill_queue_tile_parent_class)->dispose (object);
}

static void
ahoghill_queue_tile_set_property (GObject      *object,
                                  guint         prop_id,
                                  const GValue *value,
                                  GParamSpec   *pspec)
{
    AhoghillQueueTile *tile = (AhoghillQueueTile *) object;

    switch (prop_id) {

    case PROP_ITEM:
        ahoghill_queue_tile_set_item (tile, g_value_get_object (value));
        break;

    default:
        break;
    }
}

static void
ahoghill_queue_tile_get_property (GObject    *object,
                                  guint       prop_id,
                                  GValue     *value,
                                  GParamSpec *pspec)
{
    switch (prop_id) {

    case PROP_ITEM:
        break;

    default:
        break;
    }
}

static gboolean
ahoghill_queue_tile_button_release (ClutterActor       *actor,
                                    ClutterButtonEvent *event)
{
    AhoghillQueueTile *tile = (AhoghillQueueTile *) actor;
    AhoghillQueueTilePrivate *priv = tile->priv;

    if (event->click_count == 2) {
        g_signal_emit (actor, signals[CLICKED], 0, priv->item);
    }

    return TRUE;
}

static void
ahoghill_queue_tile_class_init (AhoghillQueueTileClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;
    ClutterActorClass *a_class = (ClutterActorClass *) klass;

    o_class->dispose = ahoghill_queue_tile_dispose;
    o_class->finalize = ahoghill_queue_tile_finalize;
    o_class->set_property = ahoghill_queue_tile_set_property;
    o_class->get_property = ahoghill_queue_tile_get_property;

    a_class->button_release_event = ahoghill_queue_tile_button_release;

    g_type_class_add_private (klass, sizeof (AhoghillQueueTilePrivate));

    g_object_class_install_property (o_class, PROP_ITEM,
                                     g_param_spec_object ("item", "", "",
                                                          AHG_TYPE_ITEM,
                                                          G_PARAM_STATIC_STRINGS |
                                                          G_PARAM_WRITABLE));
    signals[CLICKED] = g_signal_new ("clicked",
                                     G_TYPE_FROM_CLASS (o_class),
                                     G_SIGNAL_RUN_FIRST |
                                     G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
                                     g_cclosure_marshal_VOID__OBJECT,
                                     G_TYPE_NONE, 1, AHG_TYPE_ITEM);
}

static void
ahoghill_queue_tile_init (AhoghillQueueTile *self)
{
    AhoghillQueueTilePrivate *priv;

    self->priv = GET_PRIVATE (self);
    priv = self->priv;

    clutter_actor_set_reactive ((ClutterActor *) self, TRUE);

    priv->row1 = mx_label_new ("");
    mx_stylable_set_style_class ((MxStylable *) priv->row1,
                                 "AhoghillPrimaryLabel");
    mx_table_add_actor (MX_TABLE (self),
                          (ClutterActor *) priv->row1, 0, 0);

    priv->row2 = mx_label_new ("");
    mx_stylable_set_style_class ((MxStylable *) priv->row2,
                                 "AhoghillSecondaryLabel");
    mx_table_add_actor (MX_TABLE (self),
                          (ClutterActor *) priv->row2, 1, 0);
}

static void
update_tile (AhoghillQueueTile *tile)
{
    AhoghillQueueTilePrivate *priv = tile->priv;
    const char *title = NULL, *artist = NULL;
    char *primary = NULL;

    switch (ahg_item_get_item_type (priv->item)) {
    case AHG_ITEM_TYPE_AUDIO:
        title = ahg_item_get_metadata (priv->item, AHG_ITEM_METADATA_TITLE);
        artist = ahg_item_get_metadata (priv->item, AHG_ITEM_METADATA_ARTIST);
        break;

    default:
        /* Don't need to deal with non-audio */
        return;
    }

    if (title == NULL) {
        char *basename;

        basename = g_path_get_basename (ahg_item_get_uri (priv->item));
        primary = g_uri_unescape_string (basename, NULL);
        g_free (basename);
    }

    mx_label_set_text (MX_LABEL (priv->row1), title ? title : primary);
    mx_label_set_text (MX_LABEL (priv->row2), artist ? artist : "");

    g_free (primary);
}

static void
item_changed_cb (AhgItem           *item,
                 AhoghillQueueTile *tile)
{
    update_tile (tile);
}

void
ahoghill_queue_tile_set_item (AhoghillQueueTile *tile,
                              AhgItem           *item)
{
    AhoghillQueueTilePrivate *priv = tile->priv;

    priv->item = g_object_ref (item);
    g_signal_connect (priv->item, "changed",
                      G_CALLBACK (item_changed_cb), tile);

    update_tile (tile);
}
