#include <dbus/dbus-glib.h>

#include "ahoghill-marshal.h"
#include "ahoghill-thumbnailer.h"
#include "tumbler-service-dbus-glue.h"

enum {
    PROP_0,
};

enum {
    LAST_SIGNAL,
};

struct _AhoghillThumbnailerPrivate {
    DBusGProxy *thumbnailer;

    guint32 request_id;
    GPtrArray *requests;
    GHashTable *requested;
};

typedef struct _TBRequest {
    AhoghillThumbnailerCallback callback;
    gpointer data;
    char *uri;
    char *mimetype;
} TBRequest;

#define THUMBNAILER_SERVICE "org.freedesktop.thumbnails.Thumbnailer1"
#define THUMBNAILER_PATH "/org/freedesktop/thumbnails/Thumbnailer1"
#define THUMBNAILER_INTERFACE "org.freedesktop.thumbnails.Thumbnailer1"

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), AHOGHILL_TYPE_THUMBNAILER, AhoghillThumbnailerPrivate))
G_DEFINE_TYPE (AhoghillThumbnailer, ahoghill_thumbnailer, G_TYPE_OBJECT);
static guint32 signals[LAST_SIGNAL] = {0,};

static void
ahoghill_thumbnailer_finalize (GObject *object)
{
    AhoghillThumbnailer *self = (AhoghillThumbnailer *) object;

    G_OBJECT_CLASS (ahoghill_thumbnailer_parent_class)->finalize (object);
}

static void
ahoghill_thumbnailer_dispose (GObject *object)
{
    AhoghillThumbnailer *self = (AhoghillThumbnailer *) object;

    G_OBJECT_CLASS (ahoghill_thumbnailer_parent_class)->dispose (object);
}

static void
ahoghill_thumbnailer_set_property (GObject      *object,
                                   guint         prop_id,
                                   const GValue *value,
                                   GParamSpec   *pspec)
{
    AhoghillThumbnailer *self = (AhoghillThumbnailer *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahoghill_thumbnailer_get_property (GObject    *object,
                                   guint       prop_id,
                                   GValue     *value,
                                   GParamSpec *pspec)
{
    AhoghillThumbnailer *self = (AhoghillThumbnailer *) object;

    switch (prop_id) {

    default:
        break;
    }
}

static void
ahoghill_thumbnailer_class_init (AhoghillThumbnailerClass *klass)
{
    GObjectClass *o_class = (GObjectClass *) klass;

    o_class->dispose = ahoghill_thumbnailer_dispose;
    o_class->finalize = ahoghill_thumbnailer_finalize;
    o_class->set_property = ahoghill_thumbnailer_set_property;
    o_class->get_property = ahoghill_thumbnailer_get_property;

    g_type_class_add_private (klass, sizeof (AhoghillThumbnailerPrivate));
}

static char *
generate_thumbnail_path (const char *uri)
{
    GChecksum *checksum;
    char *retval;

    checksum = g_checksum_new (G_CHECKSUM_MD5);
    g_checksum_update (checksum, (const guchar *) uri, -1);
    retval = g_strdup_printf ("%s/.thumbnails/large/%s.png",
                              g_get_home_dir (),
                              g_checksum_get_string (checksum));
    g_checksum_free (checksum);

    return retval;
}

static void
thumbnail_error_cb (DBusGProxy           *proxy,
                    guint                 handle,
                    const char          **uris,
                    int                   code,
                    const char           *error,
                    AhoghillThumbnailer  *thumbnailer)
{
    AhoghillThumbnailerPrivate *priv = thumbnailer->priv;
    int i;

    if (uris == NULL) {
        return;
    }

    for (i = 0; uris[i]; i++) {
        TBRequest *request;

        request = g_hash_table_lookup (priv->requested, uris[i]);
        if (request == NULL) {
            g_warning ("No thumbnail requested for %s", uris[i]);
            continue;
        }

        if (request->callback) {
            request->callback (thumbnailer, uris[i], code, error,
                               request->data);
        }

        g_hash_table_remove (priv->requested, uris[i]);
        g_free (request->uri);
        g_free (request->mimetype);
        g_slice_free (TBRequest, request);
    }
}

static void
thumbnail_ready_cb (DBusGProxy           *proxy,
                    guint                 handle,
                    const char          **uris,
                    AhoghillThumbnailer  *thumbnailer)
{
    AhoghillThumbnailerPrivate *priv = thumbnailer->priv;
    int i;

    if (uris == NULL) {
        return;
    }

    for (i = 0; uris[i]; i++) {
        TBRequest *request;
        char *thumbnail_path;
        GError *error = NULL;

        request = g_hash_table_lookup (priv->requested, uris[i]);
        if (request == NULL) {
            g_warning ("No thumbnail requested for %s", uris[i]);
            continue;
        }

        if (request->callback) {
            request->callback (thumbnailer, uris[i], -1, NULL, request->data);
        }

        g_hash_table_remove (priv->requested, uris[i]);
        g_free (request->uri);
        g_free (request->mimetype);
        g_slice_free (TBRequest, request);
    }
}

static void
ahoghill_thumbnailer_init (AhoghillThumbnailer *self)
{
    AhoghillThumbnailerPrivate *priv = GET_PRIVATE (self);
    DBusGConnection *connection;
    GError *error = NULL;

    self->priv = priv;

    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (connection == NULL) {
        g_warning ("Error getting bus: %s", error->message);
        g_error_free (error);

        priv->thumbnailer = NULL;
        return;
    }

    dbus_g_object_register_marshaller (ahoghill_marshal_VOID__UINT_POINTER,
                                       G_TYPE_NONE, G_TYPE_UINT,
                                       G_TYPE_STRV, G_TYPE_INVALID);
    dbus_g_object_register_marshaller (ahoghill_marshal_VOID__UINT_POINTER_INT_STRING,
                                       G_TYPE_NONE, G_TYPE_UINT,
                                       G_TYPE_STRV, G_TYPE_INT,
                                       G_TYPE_STRING, G_TYPE_INVALID);

    priv->thumbnailer = dbus_g_proxy_new_for_name (connection,
                                                   THUMBNAILER_SERVICE,
                                                   THUMBNAILER_PATH,
                                                   THUMBNAILER_INTERFACE);
    dbus_g_proxy_add_signal (priv->thumbnailer, "Ready",
                             G_TYPE_UINT, G_TYPE_STRV, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal (priv->thumbnailer, "Ready",
                                 G_CALLBACK (thumbnail_ready_cb), self, NULL);
    dbus_g_proxy_add_signal (priv->thumbnailer, "Error",
                             G_TYPE_UINT, G_TYPE_STRV, G_TYPE_INT,
                             G_TYPE_STRING, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal (priv->thumbnailer, "Error",
                                 G_CALLBACK (thumbnail_error_cb), self, NULL);

    priv->requests = g_ptr_array_new ();
    priv->requested = g_hash_table_new (g_str_hash, g_str_equal);
}

static void
queue_reply (DBusGProxy *proxy,
             guint       OUT_handle,
             GError     *error,
             gpointer    userdata)
{
    AhoghillThumbnailer *thumbnailer = (AhoghillThumbnailer *) userdata;

    if (error) {
        g_warning ("Error queuing thumbnails: %s", error->message);
    }
}

AhoghillThumbnailer *
ahoghill_thumbnailer_get_default (void)
{
    static AhoghillThumbnailer *thumbnailer = NULL;

    if (G_UNLIKELY (thumbnailer == NULL)) {
        thumbnailer = g_object_new (AHOGHILL_TYPE_THUMBNAILER, NULL);
    }

    return thumbnailer;
}

static gboolean
process_request_queue (gpointer userdata)
{

    AhoghillThumbnailer *thumbnailer = (AhoghillThumbnailer *) userdata;
    AhoghillThumbnailerPrivate *priv = thumbnailer->priv;
    char **uris, **mimetypes;
    int i;

    uris = g_new0 (char *, priv->requests->len + 1);
    mimetypes = g_new0 (char *, priv->requests->len + 1);
    for (i = 0; i < priv->requests->len; i++) {
        TBRequest *request = priv->requests->pdata[i];
        uris[i] = request->uri;
        mimetypes[i] = request->mimetype;

        g_hash_table_insert (priv->requested, request->uri, request);
    }
    g_ptr_array_free (priv->requests, TRUE);
    priv->requests = g_ptr_array_new ();

    /* Submit requests to thumbnailer */
    org_freedesktop_thumbnails_Thumbnailer1_queue_async (priv->thumbnailer,
                                                         (const char **) uris,
                                                         (const char **) mimetypes,
                                                         "large", "foreground",
                                                         0, queue_reply,
                                                         thumbnailer);
    priv->request_id = 0;
    return FALSE;
}

void
ahoghill_thumbnailer_request_thumbnail (AhoghillThumbnailer         *thumbnailer,
                                        const char                  *uri,
                                        const char                  *mimetype,
                                        AhoghillThumbnailerCallback  callback,
                                        gpointer                     userdata)
{
    AhoghillThumbnailerPrivate *priv = thumbnailer->priv;
    TBRequest *request;

    request = g_slice_new (TBRequest);
    request->uri = g_strdup (uri);
    request->mimetype = g_strdup (mimetype);
    request->callback = callback;
    request->data = userdata;

    g_ptr_array_add (priv->requests, request);
    if (priv->request_id == 0) {
        priv->request_id = g_timeout_add (500, process_request_queue,
                                          thumbnailer);
    }
}
